<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Admin\LicenseController;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Notifications\Notifiable;
use App\Services\Statistics\UserUsageYearlyService;
use App\Services\Statistics\UserUsageMonthlyService;
use Illuminate\Support\Str;
use Illuminate\Http\UploadedFile;
use App\Models\Subscription;
use App\Models\Plan;
use App\Models\Project;
use App\Models\User;
use App\Models\Language;
use DB;

class UserController extends Controller
{
    use Notifiable;

    private $api;

    public function __construct()
    {
        $this->api = new LicenseController();
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {         
        $verify = $this->api->verify_license();

        if($verify['status']!=true){
            die('Your license is invalid or not activated, please contact support.');
        }
                
        $year = $request->input('year', date('Y'));
        $month = $request->input('month', date('m'));

        $usage_yearly = new UserUsageYearlyService($year);
        $usage_monthly = new UserUsageMonthlyService($month);

        $user_data_year = [
            'total_minutes' => $usage_yearly->getTotalMinutes(),
            'total_words' => $usage_yearly->getTotalWords(),
            'total_file_transcribe' => $usage_yearly->getTotalFileTranscribe(),
            'total_recording_transcribe' => $usage_yearly->getTotalRecordingTranscribe(),
            'total_live_transcribe' => $usage_yearly->getTotalLiveTranscribe(),
        ];

        $user_data_month = [
            'total_minutes' => $usage_monthly->getTotalMinutes(),
            'total_words' => $usage_monthly->getTotalWords(),
            'total_audio_files' => $usage_monthly->getTotalAudioFiles()
        ];
        
        $chart_data['file_minutes'] = json_encode($usage_yearly->getFileMinutesUsage());
        $chart_data['record_minutes'] = json_encode($usage_yearly->getRecordMinutesUsage());
        $chart_data['live_minutes'] = json_encode($usage_yearly->getLiveMinutesUsage());

        if (auth()->user()->hasActiveSubscription()) {
            $subscription = Subscription::where('user_id', auth()->user()->id)->where('status', 'Active')->first();
        } else {
            $subscription = false;
        }

        $user_subscription = ($subscription) ? Plan::where('id', auth()->user()->plan_id)->first() : '';   
        $language_file = Language::where('id', auth()->user()->language_file)->pluck('language')->first();
        $language_live = Language::where('id', auth()->user()->language_live)->pluck('language')->first();
    

        return view('user.profile.index', compact('chart_data', 'user_data_year', 'user_data_month', 'user_subscription', 'language_file', 'language_live'));           
    }


    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id = null)
    {   
        $languages_file = DB::table('languages')
                ->where('type', 'file')            
                ->orWhere('type', 'both')            
                ->where('status', 'active')            
                ->orderBy('language', 'asc')
                ->get();

        $languages_live = DB::table('languages')           
                ->orWhere('type', 'both')            
                ->where('status', 'active')            
                ->orderBy('language', 'asc')
                ->get();
        
        $projects = Project::where('user_id', auth()->user()->id)->orderBy('name', 'asc')->get();

        return view('user.profile.edit', compact('languages_file', 'languages_live', 'projects'));
    }

    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(User $user)
    {           
        $user->update(request()->validate([
            'name' => 'required|string|max:255',
            'email' => ['required','string','email','max:255',Rule::unique('users')->ignore($user)],
            'job_role' => 'nullable|string|max:255',
            'company' => 'nullable|string|max:255',
            'website' => 'nullable|string|max:255',
            'phone_number' => 'nullable|max:20',
            'address' => 'nullable|string|max:255',            
            'city' => 'nullable|string|max:255',
            'postal_code' => 'nullable|string|max:255',
            'country' => 'string|max:255',
            'language_file' => 'nullable',
            'language_live' => 'nullable',
            'default_project' => 'nullable',
        ]));
        
        if (request()->has('profile_photo')) {

            request()->validate([
                'profile_photo' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:5048'
            ]);
            
            $image = request()->file('profile_photo');

            $name = Str::random(20);
         
            $folder = '/uploads/img/users/';
          
            $filePath = $folder . $name . '.' . $image->extension();
            
            $this->uploadImage($image, $folder, 'public', $name);

            $user->profile_photo_path = $filePath;

            $user->save();
        }

        return redirect()->route('user.profile.edit', compact('user'))->with('success','Profile Successfully Updated');

    }


    /**
     * Upload user profile image
     */
    public function uploadImage(UploadedFile $file, $folder = null, $disk = 'public', $filename = null)
    {
        $name = !is_null($filename) ? $filename : Str::random(25);

        $image = $file->storeAs($folder, $name .'.'. $file->extension(), $disk);

        return $image;
    }


}
